function varargout=nuget(cmd,varargin)
% code by jph

global nu user
persistent prg

switch cmd

case 'all'

  varargout{1}=cell(0,1);
  fdn=dir(['rec/' varargin{1} '*' varargin{2} '.mat']);
  for c1=1:length(fdn)
    varargout{1}=vertcat(varargout{1},fdn(c1).name(1:end-4));
  end

case 'assert'
  X=all(varargin{1});
  for c1=1:ndims(X) % don't understand matlab dim logic
    X=all(X);
  end
  if ~X
    error('assert failed')
  end
  
case 'cell'
  % returns
  %   1.  cell2mat(nu.cell) with unconnected channels removed
  %   2.  indices to channels (formerly nu.cast)

  if isfield(nu,'cell')
    if iscell(nu.cell)
      nu.now=cell2mat(nu.now);
      varargout{1}=cell2mat(nu.cell);
      num=size(varargout{1},2);
      varargout{2}=find(any(varargout{1}<1)); % remove disconnected channels
      varargout{1}=varargout{1}(:,varargout{2});
      fprintf('remark: <a href="">%i</a> unused channels removed\n',num-length(varargout{2}))
    else
      varargout{1}=nu.cell;
      varargout{2}=nu.cast;
      disp('remark: cell already converted to matrix')
    end
  end

case 'connected.always'
  res=ones(1,40);
  for c1=1:nu.numel
    res=res&all(nu.cell{c1}<1);
  end
  varargout{1}=find(res);
  
  
case 'channel.extract'
  col=cell(nu.numel,1);
  for c1=1:nu.numel
    col{c1}=nu.cell{c1}(:,varargin{1});
  end
  varargout{1}=cell2mat(col);
  varargout{2}=-200:1:size(varargout{1},1)-201;
  varargout{2}=1e-3*varargout{2}';
  
case 'channel.41.append'
  varargout{1}=[varargin{1} zeros(size(varargin{1},1),1)];

case 'closereq'
  % used by nu_setup to terminate setup procedure
  % hopefully calls CloseRequestFcn of 'right' figure!
  nu.run=0;
  closereq;
  varargout{1}='closereq';
  
case 'color'

  map=[
  255, 10, 10     % r
  10, 255, 10     % g
  10, 10, 255     % b
  220, 110, 0     % r1
  0, 220, 110     % g1
  110, 0, 220     % b1
  200, 200, 0     % r2
  0, 200, 200     % g2
  200, 0, 200     % b2
  110, 220, 0     % r3
  0, 110, 220     % g3
  220, 0, 110     % b3
  139, 224, 70    % r4
  70, 139, 224    % g4
  224, 70, 139    % b4
  133, 192, 141   % r5
  141, 133, 192   % g5
  192, 141, 133   % b5
  0, 255, 86      % r6
  86, 0, 255      % g6
  255, 86, 0      % b6
  188, 191, 51    % r7
  51, 188, 191    % g7
  191, 51, 188    % b7
  ];
  
  idx=mod(max(varargin{1}-1,0),size(map,1))+1;
  varargout{1}=map(idx,:)/255;

case 'desktop'
  % does not work under all circumstances :-(
  varargout{1}=system('desktop.scf');

case 'decimate'
  num=size(varargin{1},2);
  col=cell(1,num);
  for c1=1:num
    col{c1}=decimate(varargin{1}(:,c1),varargin{2});
  end
  varargout{1}=cell2mat(col);  
  varargout{2}=-200:varargin{2}:size(varargin{1},1)-201;
  varargout{2}=(1e-3)*varargout{2}';
  
case 'discretize.now'

  ind=floor(varargin{1}*1000+200);
  varargout{1}=floor(ind/200)+1;
  varargout{2}=mod(ind,200)+1;
  
  fnd=find(nu.numel<varargout{1});
  varargout{1}(fnd)=nu.numel;
  varargout{2}(fnd)=200;
  
%case 'discretize.now'
%
%  ind=floor(varargin{1}*1000+200);
%  varargout{1}=floor(ind/200)+1;
%  varargout{2}=mod(ind,200)+1;
%  
%  fnd=find(nu.numel<varargout{1});
%  varargout{1}(fnd)=nu.numel;
%  varargout{2}(fnd)=200;
  
case 'events.evaluated'
  col=cell(0);
  for c1=1:length(nu.events)
    if ismember(nu.events{c1}.eval,varargin{1})
      if isfield(nu.events{c1},varargin{2}) % when prg has aborted before completion
        val=getfield(nu.events{c1},varargin{2});
        if numel(val)==varargin{3}
          col=vertcat(col,val);
        end
      end
    end
  end
  varargout{1}=cell2mat(col);
  
case 'filename'
  str=datestr(nu.clock,30);
  if isstruct(user)
    if isfield(user,'title')
      str=[str '_' user.title];
    end
  end
  str=[str '_' nu.varargin{1}];
  tmp=nuget('varargin.struct','title',[]);
  if ~isempty(tmp)
    str=[str '_' tmp];
  end
  str(find(~ismember(str,' (),-.0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ[]_abcdefghijklmnopqrstuvwxyz')))=[];
  str(find(str==' '))='-';
  varargout{1}=str;
  
case 'filter'
  varargout{1}=cell(length(varargin{1}),1);
  for c1=1:length(varargin{1})
    fX=fft(nu.cell{varargin{1}(c1)});
    fX([1+varargin{2} end-varargin{2}+1],:)=0;
    %sum(abs(imag(ifft(fX))))
    varargout{1}{c1}=ifft(fX);
  end

case 'ground'
  % this is equivalent to the matrix multipliation of id + one column -1 ...
  varargout{1}=varargin{1}-repmat(varargin{1}*varargin{2}',[1 size(varargin{1},2)]);
  
case 'load'
  % loads nu file only once (much quicker when developping code)
  % calling function must declare nu global
  
  if exist('nu')
    if isfield(nu,'load')
      if strcmp(nu.load,varargin{1})
        disp('remark: data already in memory')
        return % get out of here
      end
    end
    nu=0; % ensure data integrity
  end
  disp('remark: load file from disc')
  load(['rec/' varargin{1} '.mat'])
  nu.load=varargin{1};
  
case 'plot.trigger.struct'

  plot(varargin{1}.XData,varargin{1}.YData)
  hold on
  plot(varargin{1}.toc,varargin{1}.mean,'r*')
  hold off

case 'remove.polynom'
  dom=reshape(linspace(-1,1,numel(varargin{1})),size(varargin{1}));
  pol=polyfit(dom,varargin{1},varargin{2});
  varargout{1}=varargin{1}-polyval(pol,dom);
  %if ismember('normalize')
  for c1=3:nargin-1
    switch varargin{c1}
      case 'normalize'
        varargout{1}=varargout{1}/(norm(varargout{1})+eps);
      case 'minmax'
        mn=min(varargout{1});
        mx=max(varargout{1});
        fac=2/(mx-mn);
        varargout{1}=(varargout{1}-mn)*fac-1;
      otherwise
        disp('remove.polynom unknown option')
    end
  end
  
case 'report'
  if isfield(nu,'load')
    fprintf('\tname\t%s\n',nu.load)
  end
  fprintf('\tmouse\t%i\tbutton\t%i\tkeyboard\t%i\n',numel(nu.trace.mouse),numel(nu.trace.print),numel(nu.trace.keyboard))
  fprintf('\tlast event at %f sec\n',nu.now{end})
  
case 'triggered.flash'
  % TODO narrow intervals down use except only for median computation
  [SYN,tim]=nuget('grab.block',varargin{1}+[0 0.799],[nu.light.channel nu.sound.channel]);
  snd=SYN(:,2);
  SYN=convn(SYN(:,1),ones(7,1)/7,'same');
  mup=median(SYN);
  ind=6;
  SYN([1:ind end-ind:end])=mup;
  mdn=median(SYN(find(SYN<mup)));
  lam=8;
  mup=mup*lam+mdn*(1-lam);
  fnd=find(mup<SYN);
  if isempty(fnd)
    varargout{1}=[];
    varargout{2}=[];
  else
    varargout{1}=tim(fnd(1));
    varargout{2}=struct('tic',varargin{1},...
                        'toc',varargout{1},...
                        'mean',mup,...
                        'XData',tim,...
                        'YData',SYN,...
                        'ZData',snd);
  end
  
case 'triggered.flashes'
  % TODO merge !!!
  varargout{1}={};
  varargout{2}={};
  varargout{3}={};
  for ins=reshape(varargin{1},[1 numel(varargin{1})])
    % TODO check if data outside !!!
    [SYN,tim]=nuget('grab.block',ins+[0 0.799],nu.light.channel);
    SYN=convn(SYN,ones(7,1)/7,'same');
    mup=median(SYN);
    ind=6;
    SYN([1:ind end-ind:end])=mup;
    mdn=median(SYN(find(SYN<mup)));
    lam=8;
    mup=mup*lam+mdn*(1-lam);
    fnd=find(mup<SYN);
    if ~isempty(fnd)
      fnd=tim(fnd(1));
      varargout{1}=vertcat(varargout{1},fnd);
      varargout{2}=vertcat(varargout{2},...
                      struct('tic',ins,...
                             'toc',fnd,...
                             'mean',mup,...
                             'XData',tim,...
                             'YData',SYN));
      varargout{3}=cat(3,varargout{3},nuget('grab.blocks.discrete',fnd+varargin{2},varargin{3}));      
    end
  end
  varargout{1}=cell2mat(varargout{1});
  varargout{2}=cell2mat(varargout{2});
  varargout{3}=cell2mat(varargout{3});
  
case 'grab.blocks.discrete'
  [pck,ext]=nuget('discretize.now',varargin{1});
  col={ nu.cell{pck}(ext(1):end,:) };
  num=size(col{1},1);
  while num<=varargin{2}
    pck=pck+1;
    col=vertcat(col,nu.cell{pck});
    num=num+size(col{end},1);
  end
  varargout{1}=cell2mat(col);
  varargout{1}=varargout{1}(1:varargin{2},:);
  varargout{2}=((pck-2)*200+ext-2+[1:varargin{2}])*1e-3;

  
case 'grab.block'
  [pck,ext]=nuget('discretize.now',varargin{1});
  if pck(1)==pck(2)
    varargout{1}=nu.cell{pck(1)}(ext(1):ext(2),varargin{2});
    varargout{2}=(pck(1)-2)*200+ext(1):ext(2);
  else
    col={ nu.cell{pck(1)}(ext(1):end,varargin{2}) };
    for c1=pck(1)+1:pck(2)-1
      col=vertcat(col,nu.cell{c1}(:,varargin{2}));
    end
    col=vertcat(col,nu.cell{pck(2)}(1:ext(2),varargin{2}));
    varargout{1}=cell2mat(col);
  end
  varargout{2}=((pck(1)-2)*200+ext(1):(pck(2)-2)*200+ext(2))*1e-3;
  
case 'varargin.struct'
  if 1<length(varargin)
    varargout{1}=varargin{2};
  else
    varargout{1}=[];
  end
  if 1<length(nu.varargin)
    if isstruct(nu.varargin{2})
      if isfield(nu.varargin{2},varargin{1})
        varargout{1}=getfield(nu.varargin{2},varargin{1});
      end
    end
  end

case 'video.triggers'

% init=599 trig=52  78
% init=566 trig=68  90
% init=518 trig=52  75
% init=569 trig=67  89
% init=572 trig=52  76
% init=552 trig=54  76
% init=534 trig=64  87
% init=555 trig=66  89

  varargout{1}=[];
  [SYN,fin]=nuget('channel.extract',nu.synchronize);
  SYN=convn(SYN,ones(7,1)/7,'same');
  mup=median(SYN);
  mdn=median(SYN(find(SYN<mup)));
  lam=4;
  mup=mup*lam+mdn*(1-lam);
  lab=bwlabel(mup<SYN)';
  [u,U]=xunique(lab);
  u(1)=[]; U(1)=[]; % remove 0 label  
  %[u ;  U]
  %bar(u,U)
  %return
  
  trg=find(480<U); % find initial long flash (>350ms)
  if isempty(trg)
    disp('remark: no trigger signal found')        
    plot(fin,SYN)
    return
  end
  col=cell(0);
  for c1=find(45<U & U<97 & trg(1)<u)
    ind=find(lab==u(c1));
    col=horzcat(col,ind(1));
  end
  col=cell2mat(horzcat(col{1}-[1000 500],col));
  %col=col+500;
  
  if length(col)~=240
    fprintf('warning: number of triggers %i\n',length(col))
    col=col(1:min(end,240));
  end
  mean(diff(col))
  
  if ismember('plot',varargin)
    plot(fin,SYN)
    hold on
    plot(fin([1 end]),[mup mup],'r:')
    %plot(fin([1 end]),[mdn mdn],'r:')
    %plot(fin([1 end]),[lim lim],'g-')
    axis tight    
    plot(fin(col),repmat(mup+1e-5,size(col)),'g+');
    res=diff(fin(col));
    title(['mean=' num2str(mean(res)) ' var=' num2str(var(res)) ])
    xlabel('[sec]')    
  end
  varargout{1}=col;
  
case 'plot.unique'

  set(varargin{1},'Color',nuget('color',varargin{2}))
  str={'-',':','-.','--'};
  ind=min(max(1,ceil(varargin{2}/12)),numel(str));  
  set(varargin{1},'LineStyle',str{ind})
  
case 'screen.capture'

  varargout{1}=['scr/' datestr(now,30) '_' varargin{1} '.png'];
  robot=java.awt.Robot;
  toolkit=java.awt.Toolkit.getDefaultToolkit();
  javax.imageio.ImageIO.write(robot.createScreenCapture(java.awt.Rectangle(1024,768)),'png',java.io.File(varargout{1}));

case 'screen.capture.continuous'

  while 1
    disp(nuget('screen.capture',varargin{1}))
    pause(2)
  end
  
otherwise
  error('undefined option')

end

